package com.schbrain.initializer;

import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.boot.context.metrics.buffering.BufferingApplicationStartup;
import org.springframework.boot.context.metrics.buffering.StartupTimeline;
import org.springframework.boot.context.metrics.buffering.StartupTimeline.TimelineEvent;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.core.metrics.StartupStep;
import org.springframework.core.metrics.StartupStep.Tag;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@SpringBootApplication
public class Application {

    public static void main(String[] args) {
        ConfigurableApplicationContext applicationContext = new SpringApplicationBuilder()
                .applicationStartup(new BufferingApplicationStartup(Integer.MAX_VALUE))
                .sources(Application.class)
                .run(args);
        BufferingApplicationStartup applicationStartup = (BufferingApplicationStartup) applicationContext.getApplicationStartup();
        StartupTimeline startupTimeline = applicationStartup.drainBufferedTimeline();
        for (TimelineEvent event : startupTimeline.getEvents()) {
            StartupStep step = event.getStartupStep();
            StringBuilder message = new StringBuilder("step: {}");
            List<Object> values = new ArrayList<>();
            values.add(step.getName());
            for (Tag tag : step.getTags()) {
                message.append(" tag: {}, {}");
                values.add(tag.getKey());
                values.add(tag.getValue());
            }
            log.info(message.toString(), values.toArray());
        }
    }

}
